@@ -92,8 +92,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define MOUSE_PREFS		"mouse"
 #define SOUND_PREFS		"sound"
 
-#define ICON_SIZE 128
-
 /* We'll try to connect to the device repeatedly for that
  * amount of time before we bail out */
 #define CONNECT_TIMEOUT 3.0
@@ -111,19 +109,23 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define DEVICES_PAGE                 "devices-page"
 
 enum {
-	CONNECTING_NOTEBOOK_PAGE_SWITCH = 0,
-	CONNECTING_NOTEBOOK_PAGE_SPINNER = 1
+	CONNECTING_STATE_IDLE = 0,
+	CONNECTING_STATE_LOADING = 1
 };
 
 static void
-set_connecting_page (BluetoothSettingsWidget *self,
-		     int                      page)
+set_connecting_state (BluetoothSettingsWidget *self,
+		     int                      state)
 {
-	if (page == CONNECTING_NOTEBOOK_PAGE_SPINNER)
+	if (state == CONNECTING_STATE_LOADING) {
 		gtk_spinner_start (GTK_SPINNER (WID ("connecting_spinner")));
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (WID ("connecting_notebook")), page);
-	if (page == CONNECTING_NOTEBOOK_PAGE_SWITCH)
+		gtk_widget_set_visible (GTK_WIDGET (WID ("connecting_spinner")), TRUE);
+		gtk_widget_set_sensitive (GTK_WIDGET (WID ("switch_connection")), FALSE);
+	} else {
 		gtk_spinner_stop (GTK_SPINNER (WID ("connecting_spinner")));
+		gtk_widget_set_visible (GTK_WIDGET (WID ("connecting_spinner")), FALSE);
+		gtk_widget_set_sensitive (GTK_WIDGET (WID ("switch_connection")), TRUE);
+	}
 }
 
 static void
@@ -162,6 +164,7 @@ has_default_adapter (BluetoothSettingsWidget *self)
 typedef struct {
 	char             *bdaddr;
 	BluetoothSettingsWidget *self;
+	gboolean          state;
 } ConnectData;
 
 static void
@@ -188,12 +191,15 @@ connect_done (GObject      *source_object,
 		button = GTK_SWITCH (WID ("switch_connection"));
 
 		/* Ensure the switch position is in the correct place. */
-		gtk_switch_set_active (button, gtk_switch_get_state (button));
-
-		if (success == FALSE) {
+		if (success) {
+			gtk_switch_set_state (button, data->state);
+			gtk_switch_set_active (button, data->state);
+		} else {
+			gtk_switch_set_state (button, !data->state);
+			gtk_switch_set_active (button, !data->state);
 			g_debug ("Connection failed to %s: %s", data->bdaddr, error->message);
 		}
-		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
+		set_connecting_state (self, CONNECTING_STATE_IDLE);
 	}
 
 	remove_connecting (self, data->bdaddr);
@@ -882,6 +888,9 @@ connect_callback (GObject      *source_object,
 			data->timeout_id = g_timeout_add (500, connect_timeout_cb, data);
 			return;
 		}
+
+		set_connecting_state (data->self, CONNECTING_STATE_IDLE);
+
 		g_debug ("Failed to connect to device %s: %s", data->device, error->message);
 	}
 
@@ -1062,6 +1071,7 @@ switch_connected_state_set (GtkSwitch               *button,
 	data = g_new0 (ConnectData, 1);
 	data->bdaddr = g_strdup (self->selected_bdaddr);
 	data->self = self;
+	data->state = state;
 
 	if (gtk_switch_get_active (button))
 		g_object_set (G_OBJECT (self->client),
@@ -1075,7 +1085,7 @@ switch_connected_state_set (GtkSwitch               *button,
 					  data);
 
 	add_connecting (self, data->bdaddr);
-	set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SPINNER);
+	set_connecting_state (self, CONNECTING_STATE_LOADING);
 
 	return TRUE;
 }
@@ -1109,59 +1119,60 @@ update_properties (BluetoothSettingsWidget *self,
 	self->selected_object_path = g_strdup (bluetooth_device_get_object_path (device));
 
 	/* Hide all the buttons now, and show them again if we need to */
-	gtk_widget_hide (WID ("keyboard_button"));
-	gtk_widget_hide (WID ("sound_button"));
-	gtk_widget_hide (WID ("mouse_button"));
-	gtk_widget_hide (WID ("send_button"));
+	gtk_widget_set_visible (WID ("keyboard_button"), FALSE);
+	gtk_widget_set_visible (WID ("sound_button"), FALSE);
+	gtk_widget_set_visible (WID ("mouse_button"), FALSE);
+	gtk_widget_set_visible (WID ("send_button"), FALSE);
 
 	/* Name */
-	gtk_window_set_title (GTK_WINDOW (self->properties_dialog), alias);
+	adw_status_page_set_title (ADW_STATUS_PAGE (WID ("status_page")), alias);
 	g_free (self->selected_name);
 	self->selected_name = alias;
 
 	/* Icon */
-	gtk_image_set_from_icon_name (GTK_IMAGE (WID ("image")), icon);
+	adw_status_page_set_icon_name (ADW_STATUS_PAGE (WID ("status_page")), icon);
 
 	/* Connection */
 	button = GTK_SWITCH (WID ("switch_connection"));
 
 	gtk_switch_set_state (button, connected);
+	gtk_switch_set_active (button, connected);
 	if (is_connecting (self, bdaddr)) {
-		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SPINNER);
+		set_connecting_state (self, CONNECTING_STATE_LOADING);
 	} else {
-		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
+		set_connecting_state (self, CONNECTING_STATE_IDLE);
 	}
 
 	/* Paired */
-	gtk_label_set_text (GTK_LABEL (WID ("paired_label")),
-			    paired ? _("Yes") : _("No"));
+	adw_action_row_set_subtitle (ADW_ACTION_ROW (WID ("paired_row")),
+				     paired ? _("Yes") : _("No"));
 
 	/* UUIDs */
-	gtk_widget_set_sensitive (GTK_WIDGET (button), connectable);
-	if (uuids && g_strv_contains (uuids, "OBEXObjectPush"))
-		gtk_widget_show (WID ("send_button"));
+	if (uuids && g_strv_contains ((const char * const*) uuids, "OBEXObjectPush"))
+		gtk_widget_set_visible (WID ("send_button"), TRUE);
 
 	/* Type */
-	gtk_label_set_text (GTK_LABEL (WID ("type_label")), bluetooth_type_to_string (type));
+	adw_action_row_set_subtitle (ADW_ACTION_ROW (WID ("type_row")),
+				     bluetooth_type_to_string (type));
 	switch (type) {
 	case BLUETOOTH_TYPE_KEYBOARD:
-		gtk_widget_show (WID ("keyboard_button"));
+		gtk_widget_set_visible (WID ("keyboard_button"), TRUE);
 		break;
 	case BLUETOOTH_TYPE_MOUSE:
 	case BLUETOOTH_TYPE_TABLET:
-		gtk_widget_show (WID ("mouse_button"));
+		gtk_widget_set_visible (WID ("mouse_button"), TRUE);
 		break;
 	case BLUETOOTH_TYPE_HEADSET:
 	case BLUETOOTH_TYPE_HEADPHONES:
 	case BLUETOOTH_TYPE_OTHER_AUDIO:
-		gtk_widget_show (WID ("sound_button"));
+		gtk_widget_set_visible (WID ("sound_button"), TRUE);
 	default:
 		/* others? */
 		;
 	}
 
 	/* Address */
-	gtk_label_set_text (GTK_LABEL (WID ("address_label")), bdaddr);
+	adw_action_row_set_subtitle (ADW_ACTION_ROW (WID ("address_row")), bdaddr);
 
 	g_free (self->selected_bdaddr);
 	self->selected_bdaddr = bdaddr;
@@ -1176,28 +1187,25 @@ switch_panel (BluetoothSettingsWidget *self,
 		       0, panel);
 }
 
-static gboolean
+static void
 keyboard_callback (GtkButton        *button,
 		   BluetoothSettingsWidget *self)
 {
 	switch_panel (self, KEYBOARD_PREFS);
-	return TRUE;
 }
 
-static gboolean
+static void
 mouse_callback (GtkButton        *button,
 		BluetoothSettingsWidget *self)
 {
 	switch_panel (self, MOUSE_PREFS);
-	return TRUE;
 }
 
-static gboolean
+static void
 sound_callback (GtkButton        *button,
 		BluetoothSettingsWidget *self)
 {
 	switch_panel (self, SOUND_PREFS);
-	return TRUE;
 }
 
 static void
@@ -1282,18 +1290,14 @@ remove_selected_device (BluetoothSettingsWidget *self)
 }
 
 static void
-confirm_dialog_response_cb (GtkDialog *dialog,
-			    gint       response,
+confirm_dialog_remove_cb (AdwMessageDialog *dialog,
+			    gchar     *response,
 			    gpointer   user_data)
 {
 	BluetoothSettingsWidget *self = user_data;
 
-	if (response == GTK_RESPONSE_ACCEPT) {
-		remove_selected_device (self);
-		gtk_widget_hide (GTK_WIDGET (self->properties_dialog));
-	}
-
-	gtk_window_destroy (GTK_WINDOW (dialog));
+	remove_selected_device (self);
+	gtk_widget_set_visible (GTK_WIDGET (self->properties_dialog), FALSE);
 }
 
 static void
@@ -1302,20 +1306,20 @@ show_confirm_dialog (BluetoothSettingsWidget *self,
 {
 	GtkWidget *dialog;
 
-	dialog = gtk_message_dialog_new (GTK_WINDOW (self->properties_dialog),
-					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+	dialog = adw_message_dialog_new (GTK_WINDOW (self->properties_dialog), NULL,
+					 _("If you remove the device, you will have to set it up again before next use."));
+	adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog),
 					 _("Remove “%s” from the list of devices?"), name);
-	g_object_set (G_OBJECT (dialog), "secondary-text",
-		      _("If you remove the device, you will have to set it up again before next use."),
-		      NULL);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Remove"), GTK_RESPONSE_ACCEPT);
+	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+				      "cancel", _("_Cancel"),
+				      "remove", _("_Remove"),
+				      NULL);
+	adw_message_dialog_set_response_appearance (ADW_MESSAGE_DIALOG (dialog), "remove", ADW_RESPONSE_DESTRUCTIVE);
 
-	g_signal_connect (dialog, "response", G_CALLBACK (confirm_dialog_response_cb), self);
+	g_signal_connect (dialog, "response::remove", G_CALLBACK (confirm_dialog_remove_cb), self);
 
-	gtk_widget_show (dialog);
+	gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static void
@@ -1598,13 +1602,7 @@ devices_coldplug (BluetoothSettingsWidget *self)
 static void
 setup_properties_dialog (BluetoothSettingsWidget *self)
 {
-	GtkStyleContext *context;
-
-	self->properties_dialog = g_object_new (GTK_TYPE_DIALOG, "use-header-bar", TRUE, NULL);
-	gtk_window_set_hide_on_close (self->properties_dialog, TRUE);
-	gtk_widget_set_size_request (GTK_WIDGET (self->properties_dialog), 380, -1);
-	gtk_window_set_resizable (self->properties_dialog, FALSE);
-	gtk_window_set_child (self->properties_dialog, WID ("properties_vbox"));
+	self->properties_dialog = GTK_WINDOW (WID ("properties_dialog"));
 
 	g_signal_connect (G_OBJECT (WID ("delete_button")), "clicked",
 			  G_CALLBACK (delete_clicked), self);
@@ -1618,12 +1616,6 @@ setup_properties_dialog (BluetoothSettingsWidget *self)
 			  G_CALLBACK (send_callback), self);
 	g_signal_connect (G_OBJECT (WID ("switch_connection")), "state-set",
 			  G_CALLBACK (switch_connected_state_set), self);
-
-	/* Styling */
-	gtk_image_set_pixel_size (GTK_IMAGE (WID ("image")), ICON_SIZE);
-
-	context = gtk_widget_get_style_context (WID ("delete_button"));
-	gtk_style_context_add_class (context, "destructive-action");
 }
 
 static void
